@@ -25,11 +25,11 @@ import org.testng.annotations.IConfigurationAnnotation;
 import org.testng.annotations.NoInjection;
 import org.testng.collections.Lists;
 import org.testng.collections.Maps;
+import org.testng.collections.Sets;
 import org.testng.internal.InvokeMethodRunnable.TestNGRuntimeException;
 import org.testng.internal.ParameterHolder.ParameterOrigin;
 import org.testng.internal.annotations.AnnotationHelper;
 import org.testng.internal.annotations.IAnnotationFinder;
-import org.testng.internal.annotations.Sets;
 import org.testng.internal.invokers.InvokedMethodListenerInvoker;
 import org.testng.internal.invokers.InvokedMethodListenerMethod;
 import org.testng.internal.thread.ThreadExecutionException;
@@ -187,8 +187,6 @@ public class Invoker implements IInvoker {
           configurationAnnotation = AnnotationHelper.findConfiguration(m_annotationFinder, method);
 
           if (MethodHelper.isEnabled(configurationAnnotation)) {
-            boolean isClassConfiguration = isClassConfiguration(configurationAnnotation);
-            boolean isSuiteConfiguration = isSuiteConfiguration(configurationAnnotation);
             boolean alwaysRun= isAlwaysRun(configurationAnnotation);
 
             if (!confInvocationPassed(tm, currentTestMethod, testClass, instance) && !alwaysRun) {
@@ -213,7 +211,7 @@ public class Invoker implements IInvoker {
             runConfigurationListeners(testResult, true /* before */);
 
             invokeConfigurationMethod(newInstance, tm,
-              parameters, isClassConfiguration, isSuiteConfiguration, testResult);
+              parameters, testResult);
 
             // TODO: probably we should trigger the event for each instance???
             testResult.setEndMillis(System.currentTimeMillis());
@@ -263,34 +261,6 @@ public class Invoker implements IInvoker {
     runConfigurationListeners(testResult, false /* after */);
   }
 
-  /**
-   * Is the current <code>IConfiguration</code> a class-level method.
-   */
-  private  boolean isClassConfiguration(IConfigurationAnnotation configurationAnnotation) {
-    if (null == configurationAnnotation) {
-      return false;
-    }
-
-    boolean before = configurationAnnotation.getBeforeTestClass();
-    boolean after = configurationAnnotation.getAfterTestClass();
-
-    return before || after;
-  }
-
-  /**
-   * Is the current <code>IConfiguration</code> a suite level method.
-   */
-  private  boolean isSuiteConfiguration(IConfigurationAnnotation configurationAnnotation) {
-    if (null == configurationAnnotation) {
-      return false;
-    }
-
-    boolean before = configurationAnnotation.getBeforeSuite();
-    boolean after = configurationAnnotation.getAfterSuite();
-
-    return before || after;
-  }
-
   /**
    * Is the <code>IConfiguration</code> marked as alwaysRun.
    */
@@ -503,7 +473,6 @@ public class Invoker implements IInvoker {
    * @param targetInstance the instance to invoke the configuration method on
    * @param tm the configuration method
    * @param params the parameters needed for method invocation
-   * @param isClass flag if the configuration method is a class level method // FIXME: this looks like a missusage
    * @param testResult
    * @throws InvocationTargetException
    * @throws IllegalAccessException
@@ -511,8 +480,6 @@ public class Invoker implements IInvoker {
   private void invokeConfigurationMethod(Object targetInstance,
                                          ITestNGMethod tm,
                                          Object[] params,
-                                         boolean isClass,
-                                         boolean isSuite,
                                          ITestResult testResult)
     throws InvocationTargetException, IllegalAccessException
   {
@@ -523,8 +490,6 @@ public class Invoker implements IInvoker {
       InvokedMethod invokedMethod= new InvokedMethod(targetInstance,
                                           tm,
                                           params,
-                                          false, /* isTest */
-                                          isClass, /* ??? */
                                           System.currentTimeMillis(),
                                           testResult);
 
@@ -656,8 +621,6 @@ public class Invoker implements IInvoker {
       invokedMethod= new InvokedMethod(instance,
           tm,
           parameterValues,
-          true /* isTest */,
-          false /* isConfiguration */,
           System.currentTimeMillis(),
           testResult);
 
@@ -833,12 +796,12 @@ public class Invoker implements IInvoker {
         // If we have parameters, set the boolean if we are about to run
         // the last invocation
         if (tm.getParameterInvocationCount() > 0) {
-          isLast = current == tm.getParameterInvocationCount();
+          isLast = current == tm.getParameterInvocationCount() * tm.getTotalInvocationCount();
         }
         // If we have invocationCount > 1, set the boolean if we are about to
         // run the last invocation
-        else if (tm.getInvocationCount() > 1) {
-          isLast = current == tm.getInvocationCount();
+        else if (tm.getTotalInvocationCount() > 1) {
+          isLast = current == tm.getTotalInvocationCount();
         }
         if (! cm.isLastTimeOnly() || (cm.isLastTimeOnly() && isLast)) {
           result.add(m);
@@ -1014,6 +977,7 @@ public class Invoker implements IInvoker {
     final FailureContext failure = new FailureContext();
     failure.count = failureCount;
     do {
+      failure.instances = Lists.newArrayList ();
       Map<String, String> allParameters = Maps.newHashMap();
       /**
        * TODO: This recreates all the parameters every time when we only need
@@ -1517,7 +1481,7 @@ public class Invoker implements IInvoker {
       return true;
     } else {
       final String message = ite.getMessage();
-      return message != null && Pattern.matches(messageRegExp, message);
+      return message != null && Pattern.compile(messageRegExp, Pattern.DOTALL).matcher(ite.getMessage()).matches();
     }
   }
 
