@@ -63,6 +63,7 @@ public class XmlSuite implements Serializable, Cloneable {
   private String m_parallel = DEFAULT_PARALLEL;
 
   private String m_parentModule = "";
+  private String m_guiceStage = "";
 
   /** Whether to SKIP or CONTINUE to re-attempt failed configuration methods. */
   public static String DEFAULT_CONFIG_FAILURE_POLICY = SKIP;
@@ -163,6 +164,10 @@ public class XmlSuite implements Serializable, Cloneable {
     return m_parentModule;
   }
 
+  public String getGuiceStage() {
+    return m_guiceStage;
+  }
+
   public ITestObjectFactory getObjectFactory() {
     return m_objectFactory;
   }
@@ -183,6 +188,10 @@ public class XmlSuite implements Serializable, Cloneable {
     m_parentModule = parentModule;
   }
 
+  public void setGuiceStage(String guiceStage) {
+    m_guiceStage = guiceStage;
+  }
+
   /**
    * Sets the configuration failure policy.
    * @param configFailurePolicy the config failure policy
@@ -458,6 +467,9 @@ public class XmlSuite implements Serializable, Cloneable {
     if (isStringNotEmpty(m_parentModule)) {
       p.setProperty("parent-module", getParentModule());
     }
+    if (isStringNotEmpty(m_guiceStage)) {
+      p.setProperty("guice-stage", getGuiceStage());
+    }
     XmlUtils.setProperty(p, "allow-return-values", String.valueOf(getAllowReturnValues()),
         DEFAULT_ALLOW_RETURN_VALUES.toString());
     xsb.push("suite", p);
@@ -587,6 +599,7 @@ public class XmlSuite implements Serializable, Cloneable {
     result.setListeners(getListeners());
     result.setParallel(getParallel());
     result.setParentModule(getParentModule());
+    result.setGuiceStage(getGuiceStage());
     result.setConfigFailurePolicy(getConfigFailurePolicy());
     result.setThreadCount(getThreadCount());
     result.setDataProviderThreadCount(getDataProviderThreadCount());
@@ -599,6 +612,7 @@ public class XmlSuite implements Serializable, Cloneable {
     result.setSkipFailedInvocationCounts(skipFailedInvocationCounts());
     result.setObjectFactory(getObjectFactory());
     result.setAllowReturnValues(getAllowReturnValues());
+    result.setTimeOut(getTimeOut());
     return result;
   }
 
